@@ -298,6 +298,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class,
     MAKE_SIGNAL(NV_CTRL_FRAMELOCK_MULTIPLY_DIVIDE_VALUE);
     MAKE_SIGNAL(NV_CTRL_FRAMELOCK_MULTIPLY_DIVIDE_MODE);
     MAKE_SIGNAL(NV_CTRL_NUMBER_OF_HARDWARE_HEADS_USED);
+    MAKE_SIGNAL(NV_CTRL_DISPLAYPORT_FORCE_ENABLE_FEC);
 #undef MAKE_SIGNAL
 
     /*
@@ -307,7 +308,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class,
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_DISPLAYPORT_LINK_RATE_10MHZ
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_DISPLAYPORT_FORCE_ENABLE_FEC
 #warning "There are attributes that do not emit signals!"
 #endif
 
